@@ -1,38 +1,23 @@
 /*
- * Copyright (c) 2000 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <libxfs.h>
+#include <xfs/libxfs.h>
 #include "command.h"
-#include "data.h"
 #include "type.h"
 #include "faddr.h"
 #include "fprint.h"
@@ -43,7 +28,7 @@
 #include "block.h"
 #include "bit.h"
 #include "output.h"
-#include "mount.h"
+#include "init.h"
 
 static int	inode_a_bmbt_count(void *obj, int startoff);
 static int	inode_a_bmx_count(void *obj, int startoff);
@@ -100,6 +85,7 @@ const field_t	inode_core_flds[] = {
 	  FLD_COUNT, TYP_NONE },
 	{ "uid", FLDT_UINT32D, OI(COFF(uid)), C1, 0, TYP_NONE },
 	{ "gid", FLDT_UINT32D, OI(COFF(gid)), C1, 0, TYP_NONE },
+	{ "flushiter", FLDT_UINT16D, OI(COFF(flushiter)), C1, 0, TYP_NONE },
 	{ "atime", FLDT_TIMESTAMP, OI(COFF(atime)), C1, 0, TYP_NONE },
 	{ "mtime", FLDT_TIMESTAMP, OI(COFF(mtime)), C1, 0, TYP_NONE },
 	{ "ctime", FLDT_TIMESTAMP, OI(COFF(ctime)), C1, 0, TYP_NONE },
@@ -122,6 +108,36 @@ const field_t	inode_core_flds[] = {
 	{ "realtime", FLDT_UINT1,
 	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_REALTIME_BIT - 1), C1,
 	  0, TYP_NONE },
+	{ "immutable", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_IMMUTABLE_BIT-1), C1,
+	  0, TYP_NONE },
+	{ "append", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_APPEND_BIT - 1), C1,
+	  0, TYP_NONE },
+	{ "sync", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_SYNC_BIT - 1), C1,
+	  0, TYP_NONE },
+	{ "noatime", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_NOATIME_BIT - 1), C1,
+	  0, TYP_NONE },
+	{ "nodump", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_NODUMP_BIT - 1), C1,
+	  0, TYP_NONE },
+	{ "rtinherit", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_RTINHERIT_BIT-1), C1,
+	  0, TYP_NONE },
+	{ "projinherit", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_PROJINHERIT_BIT-1),C1,
+	  0, TYP_NONE },
+	{ "nosymlinks", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_NOSYMLINKS_BIT-1), C1,
+	  0, TYP_NONE },
+	{ "extsz", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_EXTSIZE_BIT-1),C1,
+	  0, TYP_NONE },
+	{ "extszinherit", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_EXTSZINHERIT_BIT-1),C1,
+	  0, TYP_NONE },
 	{ "gen", FLDT_UINT32D, OI(COFF(gen)), C1, 0, TYP_NONE },
 	{ NULL }
 };
@@ -201,9 +217,9 @@ inode_a_bmbt_count(
 	ASSERT(bitoffs(startoff) == 0);
 	ASSERT(obj == iocur_top->data);
 	dip = obj;
-	if (!XFS_DFORK_Q_ARCH(dip, ARCH_CONVERT))
+	if (!XFS_DFORK_Q(dip))
 		return 0;
-	ASSERT((char *)XFS_DFORK_APTR_ARCH(dip, ARCH_CONVERT) - (char *)dip == byteize(startoff));
+	ASSERT((char *)XFS_DFORK_APTR(dip) - (char *)dip == byteize(startoff));
 	return INT_GET(dip->di_core.di_aformat, ARCH_CONVERT) == XFS_DINODE_FMT_BTREE;
 }
 
@@ -217,9 +233,9 @@ inode_a_bmx_count(
 	ASSERT(bitoffs(startoff) == 0);
 	ASSERT(obj == iocur_top->data);
 	dip = obj;
-	if (!XFS_DFORK_Q_ARCH(dip, ARCH_CONVERT))
+	if (!XFS_DFORK_Q(dip))
 		return 0;
-	ASSERT((char *)XFS_DFORK_APTR_ARCH(dip, ARCH_CONVERT) - (char *)dip == byteize(startoff));
+	ASSERT((char *)XFS_DFORK_APTR(dip) - (char *)dip == byteize(startoff));
 	return INT_GET(dip->di_core.di_aformat, ARCH_CONVERT) == XFS_DINODE_FMT_EXTENTS ?
 		INT_GET(dip->di_core.di_anextents, ARCH_CONVERT) : 0;
 }
@@ -233,7 +249,7 @@ inode_a_count(
 
 	ASSERT(startoff == 0);
 	dip = obj;
-	return XFS_DFORK_Q_ARCH(dip, ARCH_CONVERT);
+	return XFS_DFORK_Q(dip);
 }
 
 static int
@@ -247,8 +263,8 @@ inode_a_offset(
 	ASSERT(startoff == 0);
 	ASSERT(idx == 0);
 	dip = obj;
-	ASSERT(XFS_DFORK_Q_ARCH(dip, ARCH_CONVERT));
-	return bitize((int)((char *)XFS_DFORK_APTR_ARCH(dip, ARCH_CONVERT) - (char *)dip));
+	ASSERT(XFS_DFORK_Q(dip));
+	return bitize((int)((char *)XFS_DFORK_APTR(dip) - (char *)dip));
 }
 
 static int
@@ -261,9 +277,9 @@ inode_a_sfattr_count(
 	ASSERT(bitoffs(startoff) == 0);
 	ASSERT(obj == iocur_top->data);
 	dip = obj;
-	if (!XFS_DFORK_Q_ARCH(dip, ARCH_CONVERT))
+	if (!XFS_DFORK_Q(dip))
 		return 0;
-	ASSERT((char *)XFS_DFORK_APTR_ARCH(dip, ARCH_CONVERT) - (char *)dip == byteize(startoff));
+	ASSERT((char *)XFS_DFORK_APTR(dip) - (char *)dip == byteize(startoff));
 	return INT_GET(dip->di_core.di_aformat, ARCH_CONVERT) == XFS_DINODE_FMT_LOCAL;
 }
 
@@ -281,12 +297,12 @@ inode_a_size(
 	dip = obj;
 	switch (INT_GET(dip->di_core.di_aformat, ARCH_CONVERT)) {
 	case XFS_DINODE_FMT_LOCAL:
-		asf = (xfs_attr_shortform_t *)XFS_DFORK_APTR_ARCH(dip, ARCH_CONVERT);
+		asf = (xfs_attr_shortform_t *)XFS_DFORK_APTR(dip);
 		return bitize((int)asf->hdr.totsize);
 	case XFS_DINODE_FMT_EXTENTS:
 		return (int)(INT_GET(dip->di_core.di_anextents, ARCH_CONVERT) * bitsz(xfs_bmbt_rec_t));
 	case XFS_DINODE_FMT_BTREE:
-		return bitize((int)XFS_DFORK_ASIZE_ARCH(dip, mp, ARCH_CONVERT));
+		return bitize((int)XFS_DFORK_ASIZE(dip, mp));
 	default:
 		return 0;
 	}
@@ -375,12 +391,12 @@ inode_init(void)
 typnm_t
 inode_next_type(void)
 {
-	switch (iocur_top->mode & IFMT) {
-	case IFDIR:
+	switch (iocur_top->mode & S_IFMT) {
+	case S_IFDIR:
 		return XFS_DIR_IS_V2(mp) ? TYP_DIR2 : TYP_DIR;
-	case IFLNK:
+	case S_IFLNK:
 		return TYP_SYMLINK;
-	case IFREG:
+	case S_IFREG:
 		if (iocur_top->ino == mp->m_sb.sb_rbmino)
 			return TYP_RTBITMAP;
 		else if (iocur_top->ino == mp->m_sb.sb_rsumino)
@@ -445,7 +461,7 @@ inode_u_c_count(
 	dip = obj;
 	ASSERT((char *)&dip->di_u - (char *)dip == byteize(startoff));
 	return INT_GET(dip->di_core.di_format, ARCH_CONVERT) == XFS_DINODE_FMT_LOCAL &&
-	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & IFMT) == IFREG ?
+	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & S_IFMT) == S_IFREG ?
 		(int)INT_GET(dip->di_core.di_size, ARCH_CONVERT) : 0;
 }
 
@@ -489,7 +505,7 @@ inode_u_sfdir_count(
 	dip = obj;
 	ASSERT((char *)&dip->di_u - (char *)dip == byteize(startoff));
 	return INT_GET(dip->di_core.di_format, ARCH_CONVERT) == XFS_DINODE_FMT_LOCAL &&
-	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & IFMT) == IFDIR
+	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & S_IFMT) == S_IFDIR
 	       && XFS_DIR_IS_V1(mp);
 }
 
@@ -505,7 +521,7 @@ inode_u_sfdir2_count(
 	dip = obj;
 	ASSERT((char *)&dip->di_u - (char *)dip == byteize(startoff));
 	return INT_GET(dip->di_core.di_format, ARCH_CONVERT) == XFS_DINODE_FMT_LOCAL &&
-	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & IFMT) == IFDIR &&
+	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & S_IFMT) == S_IFDIR &&
 	       XFS_DIR_IS_V2(mp);
 }
 
@@ -528,7 +544,7 @@ inode_u_size(
 	case XFS_DINODE_FMT_EXTENTS:
 		return (int)(INT_GET(dip->di_core.di_nextents, ARCH_CONVERT) * bitsz(xfs_bmbt_rec_t));
 	case XFS_DINODE_FMT_BTREE:
-		return bitize((int)XFS_DFORK_DSIZE_ARCH(dip, mp, ARCH_CONVERT));
+		return bitize((int)XFS_DFORK_DSIZE(dip, mp));
 	case XFS_DINODE_FMT_UUID:
 		return bitsz(uuid_t);
 	default:
@@ -548,7 +564,7 @@ inode_u_symlink_count(
 	dip = obj;
 	ASSERT((char *)&dip->di_u - (char *)dip == byteize(startoff));
 	return INT_GET(dip->di_core.di_format, ARCH_CONVERT) == XFS_DINODE_FMT_LOCAL &&
-	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & IFMT) == IFLNK ?
+	       (INT_GET(dip->di_core.di_mode, ARCH_CONVERT) & S_IFMT) == S_IFLNK ?
 		(int)INT_GET(dip->di_core.di_size, ARCH_CONVERT) : 0;
 }
 
@@ -586,7 +602,7 @@ set_cur_inode(
 	dip = iocur_top->data;
 	iocur_top->ino = ino;
 	iocur_top->mode = INT_GET(dip->di_core.di_mode, ARCH_CONVERT);
-	if ((iocur_top->mode & IFMT) == IFDIR)
+	if ((iocur_top->mode & S_IFMT) == S_IFDIR)
 		iocur_top->dirino = ino;
 
 	/* track updated info in ring */
