@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2001 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2001,2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <xfs/libxfs.h>
@@ -198,7 +184,7 @@ get_sb(xfs_agnumber_t agno, xfs_sb_t *sb)
 		return 0;
 	}
 
-	libxfs_xlate_sb(iocur_top->data, sb, 1, ARCH_CONVERT, XFS_SB_ALL_BITS);
+	libxfs_xlate_sb(iocur_top->data, sb, 1, XFS_SB_ALL_BITS);
 
 	if (sb->sb_magicnum != XFS_SB_MAGIC) {
 		dbprintf("bad sb magic # %#x in AG %u\n",
@@ -330,7 +316,7 @@ do_uuid(xfs_agnumber_t agno, uuid_t *uuid)
 	}
 	/* set uuid */
 	memcpy(&tsb.sb_uuid, uuid, sizeof(uuid_t));
-	libxfs_xlate_sb(iocur_top->data, &tsb, -1, ARCH_CONVERT, XFS_SB_UUID);
+	libxfs_xlate_sb(iocur_top->data, &tsb, -1, XFS_SB_UUID);
 	write_cur();
 	return uuid;
 }
@@ -483,7 +469,7 @@ do_label(xfs_agnumber_t agno, char *label)
 	memset(&tsb.sb_fname, 0, sizeof(tsb.sb_fname));
 	memcpy(&tsb.sb_fname, label, len);
 	memcpy(&lbl[0], &tsb.sb_fname, sizeof(tsb.sb_fname));
-	libxfs_xlate_sb(iocur_top->data, &tsb, -1, ARCH_CONVERT, XFS_SB_FNAME);
+	libxfs_xlate_sb(iocur_top->data, &tsb, -1, XFS_SB_FNAME);
 	write_cur();
 	return &lbl[0];
 }
@@ -548,6 +534,8 @@ version_help(void)
 "\n"
 " 'version'          - print current feature bits\n"
 " 'version extflg'   - enable unwritten extents\n"
+" 'version attr1'    - enable v1 inline extended attributes\n"
+" 'version attr2'    - enable v2 inline extended attributes\n"
 "\n"
 "The version function prints currently enabled features for a filesystem\n"
 "according to its the version field of the primary superblock.\n"
@@ -558,16 +546,17 @@ version_help(void)
 }
 
 static int
-do_version(xfs_agnumber_t agno, __uint16_t versionnum)
+do_version(xfs_agnumber_t agno, __uint16_t version, __uint32_t features)
 {
 	xfs_sb_t	tsb;
 
 	if (!get_sb(agno, &tsb))
 		return 0;
 
-	tsb.sb_versionnum = versionnum;
-	libxfs_xlate_sb(iocur_top->data, &tsb,
-			-1, ARCH_CONVERT, XFS_SB_VERSIONNUM);
+	tsb.sb_versionnum = version;
+	tsb.sb_features2 = features;
+	libxfs_xlate_sb(iocur_top->data, &tsb, -1,
+			(XFS_SB_VERSIONNUM | XFS_SBS_FEATURES2));
 	write_cur();
 	return 1;
 }
@@ -607,6 +596,10 @@ version_string(
 		strcat(s, ",EXTFLG");
 	if (XFS_SB_VERSION_HASSECTOR(sbp))
 		strcat(s, ",SECTOR");
+	if (XFS_SB_VERSION_HASMOREBITS(sbp))
+		strcat(s, ",MOREBITS");
+	if (XFS_SB_VERSION_HASATTR2(sbp))
+		strcat(s, ",ATTR2");
 	return s;
 }
 
@@ -616,6 +609,7 @@ version_f(
 	char		**argv)
 {
 	__uint16_t	version = 0;
+	__uint32_t	features = 0;
 	xfs_agnumber_t	ag;
 
 	if (argc == 2) {	/* WRITE VERSION */
@@ -647,6 +641,21 @@ version_f(
 						  XFS_SB_VERSION_EXTFLGBIT;
 				break;
 			}
+		} else if (!strcasecmp(argv[1], "attr1")) {
+			if (XFS_SB_VERSION_HASATTR2(&mp->m_sb)) {
+				if (!(mp->m_sb.sb_features2 &=
+						~XFS_SB_VERSION2_ATTR2BIT))
+					mp->m_sb.sb_versionnum &=
+						~XFS_SB_VERSION_MOREBITSBIT;
+			}
+			XFS_SB_VERSION_ADDATTR(&mp->m_sb);
+			version = mp->m_sb.sb_versionnum;
+			features = mp->m_sb.sb_features2;
+		} else if (!strcasecmp(argv[1], "attr2")) {
+			XFS_SB_VERSION_ADDATTR(&mp->m_sb);
+			XFS_SB_VERSION_ADDATTR2(&mp->m_sb);
+			version = mp->m_sb.sb_versionnum;
+			features = mp->m_sb.sb_features2;
 		} else {
 			dbprintf("%s: invalid version change command \"%s\"\n",
 				progname, argv[1]);
@@ -656,15 +665,16 @@ version_f(
 		if (version) {
 			dbprintf("writing all SBs\n");
 			for (ag = 0; ag < mp->m_sb.sb_agcount; ag++)
-				if (!do_version(ag, version)) {
+				if (!do_version(ag, version, features)) {
 					dbprintf("failed to set versionnum "
 						 "in AG %d\n", ag);
 					break;
 				}
 			mp->m_sb.sb_versionnum = version;
+			mp->m_sb.sb_features2 = features;
 		}
 	}
-	dbprintf("versionnum [0x%x] = %s\n", mp->m_sb.sb_versionnum,
-			version_string(&mp->m_sb));
+	dbprintf("versionnum [0x%x+0x%x] = %s\n", mp->m_sb.sb_versionnum,
+			mp->m_sb.sb_features2, version_string(&mp->m_sb));
 	return 0;
 }
