@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2004 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 #ifndef __LIBXFS_H__
 #define __LIBXFS_H__
@@ -40,40 +26,27 @@
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_types.h>
 #include <xfs/xfs_arch.h>
-#include <xfs/xfs_sb.h>
 #include <xfs/xfs_bit.h>
 #include <xfs/xfs_inum.h>
+#include <xfs/xfs_sb.h>
 #include <xfs/xfs_ag.h>
+#include <xfs/xfs_dir2.h>
+#include <xfs/xfs_mount.h>
 #include <xfs/xfs_da_btree.h>
 #include <xfs/xfs_bmap_btree.h>
 #include <xfs/xfs_alloc_btree.h>
 #include <xfs/xfs_ialloc_btree.h>
-#include <xfs/xfs_alloc.h>
-#include <xfs/xfs_ialloc.h>
-#include <xfs/xfs_rtalloc.h>
-#include <xfs/xfs_btree.h>
-#include <xfs/xfs_dir.h>
 #include <xfs/xfs_dir_sf.h>
-#include <xfs/xfs_dir_leaf.h>
-#include <xfs/xfs_dir2.h>
-#include <xfs/xfs_dir2_data.h>
-#include <xfs/xfs_dir2_leaf.h>
-#include <xfs/xfs_dir2_block.h>
-#include <xfs/xfs_dir2_node.h>
 #include <xfs/xfs_dir2_sf.h>
 #include <xfs/xfs_attr_sf.h>
 #include <xfs/xfs_dinode.h>
-#include <xfs/xfs_attr_leaf.h>
-#include <xfs/xfs_quota.h>
-#include <xfs/xfs_mount.h>
-#include <xfs/xfs_trans_space.h>
 #include <xfs/xfs_inode.h>
 #include <xfs/xfs_buf_item.h>
 #include <xfs/xfs_inode_item.h>
+#include <xfs/xfs_alloc.h>
+#include <xfs/xfs_btree.h>
 #include <xfs/xfs_bmap.h>
-#include <xfs/xfs_imap.h>
-#include <xfs/xfs_log.h>
-#include <xfs/xfs_log_priv.h>
+
 
 #ifndef XFS_SUPER_MAGIC
 #define XFS_SUPER_MAGIC 0x58465342
@@ -145,6 +118,7 @@ extern int	libxfs_log_header (xfs_caddr_t, uuid_t *, int, int, int,
 struct xfs_inode;
 typedef struct xfs_mount {
 	xfs_sb_t		m_sb;		/* copy of fs superblock */
+	char			*m_fsname;	/* filesystem name */
 	int			m_bsize;	/* fs logical block size */
 	xfs_agnumber_t		m_agfrotor;	/* last ag where space found */
 	xfs_agnumber_t		m_agirotor;	/* last ag dir inode alloced */
@@ -191,6 +165,7 @@ typedef struct xfs_mount {
 	int			m_dalign;	/* stripe unit */
 	int			m_swidth;	/* stripe width */
 	int			m_sinoalign;	/* stripe unit inode alignmnt */
+	int			m_attr_magicpct;/* 37% of the blocksize */
 	int			m_dir_magicpct;	/* 37% of the dir blocksize */
 	__uint8_t		m_dirversion;	/* 1 or 2 */
 	int			m_dirblksize;	/* directory block sz--bytes */
@@ -199,14 +174,19 @@ typedef struct xfs_mount {
 	xfs_dablk_t		m_dirleafblk;	/* blockno of dir non-data v2 */
 	xfs_dablk_t		m_dirfreeblk;	/* blockno of dirfreeindex v2 */
 } xfs_mount_t;
+#define	XFS_DIR_IS_V1(mp)	((mp)->m_dirversion == 1)
+#define	XFS_DIR_IS_V2(mp)	((mp)->m_dirversion == 2)
 
-#define LIBXFS_MOUNT_ROOTINOS	0x0001
-#define LIBXFS_MOUNT_DEBUGGER	0x0002
+#define LIBXFS_MOUNT_ROOTINOS		0x0001
+#define LIBXFS_MOUNT_DEBUGGER		0x0002
+#define LIBXFS_MOUNT_32BITINODES	0x0004
+#define LIBXFS_MOUNT_32BITINOOPT	0x0008
+#define LIBXFS_MOUNT_COMPAT_ATTR	0x0010
 
 extern xfs_mount_t	*libxfs_mount (xfs_mount_t *, xfs_sb_t *,
 				dev_t, dev_t, dev_t, int);
 extern void	libxfs_mount_common (xfs_mount_t *, xfs_sb_t *);
-extern void	libxfs_initialize_perag (xfs_mount_t *, int);
+extern xfs_agnumber_t	libxfs_initialize_perag (xfs_mount_t *, xfs_agnumber_t);
 extern void	libxfs_umount (xfs_mount_t *);
 extern int	libxfs_rtmount_init (xfs_mount_t *);
 extern void	libxfs_alloc_compute_maxlevels (xfs_mount_t *);
@@ -231,6 +211,7 @@ typedef struct xfs_buf {
 #define XFS_BUF_PTR(bp)			((bp)->b_addr)
 #define xfs_buf_offset(bp, offset)	(XFS_BUF_PTR(bp) + (offset))
 #define XFS_BUF_ADDR(bp)		((bp)->b_blkno)
+#define XFS_BUF_SIZE(bp)		((bp)->b_bcount)
 #define XFS_BUF_COUNT(bp)		((bp)->b_bcount)
 #define XFS_BUF_TARGET(bp)		((bp)->b_dev)
 #define XFS_BUF_SET_PTR(bp,p,cnt)	((bp)->b_addr = (char *)(p)); \
@@ -253,6 +234,12 @@ extern int	libxfs_writebuf (xfs_buf_t *, int);
 extern int	libxfs_writebuf_int (xfs_buf_t *, int);
 extern void	libxfs_putbuf (xfs_buf_t *);
 
+#define LIBXFS_BREAD	0x1
+#define LIBXFS_BWRITE	0x2
+#define LIBXFS_BZERO	0x4
+
+extern void	libxfs_iomove (xfs_buf_t *, uint, int, void *, int);
+
 
 /*
  * Transaction interface
@@ -284,6 +271,8 @@ typedef struct xfs_buf_log_item {
 
 typedef struct xfs_trans {
 	unsigned int	t_type;			/* transaction type */
+	unsigned int	t_log_res;		/* amt of log space resvd */
+	unsigned int	t_log_count;		/* count for perm log res */
 	xfs_mount_t	*t_mountp;		/* ptr to fs mount struct */
 	unsigned int	t_flags;		/* misc flags */
 	long		t_icount_delta;		/* superblock icount change */
@@ -302,7 +291,7 @@ extern void	libxfs_trans_cancel (xfs_trans_t *, int);
 extern void	libxfs_mod_sb (xfs_trans_t *, __int64_t);
 
 extern int	libxfs_trans_iget (xfs_mount_t *, xfs_trans_t *, xfs_ino_t,
-				uint, struct xfs_inode **);
+				uint, uint, struct xfs_inode **);
 extern void	libxfs_trans_iput(xfs_trans_t *, struct xfs_inode *, uint);
 extern void	libxfs_trans_ijoin (xfs_trans_t *, struct xfs_inode *, uint);
 extern void	libxfs_trans_ihold (xfs_trans_t *, struct xfs_inode *);
@@ -357,13 +346,19 @@ typedef struct xfs_inode {
 	xfs_dinode_core_t	i_d;		/* most of ondisk inode */
 } xfs_inode_t;
 
-typedef struct {
+#define LIBXFS_ATTR_ROOT	0x0002	/* use attrs in root namespace */
+#define LIBXFS_ATTR_SECURE	0x0008	/* use attrs in security namespace */
+#define LIBXFS_ATTR_CREATE	0x0010	/* create, but fail if attr exists */
+#define LIBXFS_ATTR_REPLACE	0x0020	/* set, but fail if attr not exists */
+
+typedef struct cred {
 	uid_t	cr_uid;
 	gid_t	cr_gid;
 } cred_t;
 
 extern int	libxfs_inode_alloc (xfs_trans_t **, xfs_inode_t *, mode_t,
-				ushort, xfs_dev_t, cred_t *, xfs_inode_t **);
+				nlink_t, xfs_dev_t, struct cred *,
+				struct fsxattr *, xfs_inode_t **);
 extern void	libxfs_trans_inode_alloc_buf (xfs_trans_t *, xfs_buf_t *);
 
 extern void	libxfs_idata_realloc (xfs_inode_t *, int, int);
@@ -382,6 +377,12 @@ extern void	libxfs_iput (xfs_inode_t *, uint);
 /*
  * Directory interface
  */
+#include <xfs/xfs_dir_leaf.h>
+#include <xfs/xfs_dir2_data.h>
+#include <xfs/xfs_dir2_leaf.h>
+#include <xfs/xfs_dir2_block.h>
+#include <xfs/xfs_dir2_node.h>
+
 extern void	libxfs_dir_mount (xfs_mount_t *);
 extern void	libxfs_dir2_mount (xfs_mount_t *);
 extern int	libxfs_dir_init (xfs_trans_t *, xfs_inode_t *, xfs_inode_t *);
@@ -442,20 +443,23 @@ extern int	libxfs_highbit32 (__uint32_t);
 extern int	libxfs_highbit64 (__uint64_t);
 extern uint	libxfs_da_log2_roundup (uint);
 
-extern void	libxfs_xlate_sb (void *, xfs_sb_t *, int, xfs_arch_t,
-				__int64_t);
+extern void	libxfs_xlate_sb (void *, xfs_sb_t *, int, __int64_t);
 extern void	libxfs_xlate_dinode_core (xfs_caddr_t buf,
-				xfs_dinode_core_t *, int, xfs_arch_t);
+				xfs_dinode_core_t *, int);
 
 extern int	libxfs_alloc_fix_freelist (xfs_alloc_arg_t *, int);
 extern int	libxfs_alloc_file_space (xfs_inode_t *, xfs_off_t,
 				xfs_off_t, int, int);
 
-extern xfs_dahash_t	libxfs_da_hashname (uchar_t *, int);
-extern int	libxfs_attr_leaf_newentsize (xfs_da_args_t *, int, int *);
+extern xfs_dahash_t	libxfs_da_hashname (const uchar_t *, int);
+extern int	libxfs_attr_leaf_newentsize (int, int, int, int *);
+extern int	libxfs_attr_set_int (xfs_inode_t*, const char *, int, char *,
+				int, int);
+extern int	libxfs_attr_remove_int (xfs_inode_t *, const char *, int, int);
+
 
 extern void	libxfs_bmbt_get_all (xfs_bmbt_rec_t *, xfs_bmbt_irec_t *);
-#if ARCH_CONVERT != ARCH_NOCONVERT
+#if __BYTE_ORDER != __BIG_ENDIAN
 extern void	libxfs_bmbt_disk_get_all (xfs_bmbt_rec_t *, xfs_bmbt_irec_t *);
 #else
 # define libxfs_bmbt_disk_get_all(r,s)	libxfs_bmbt_get_all(r,s)
@@ -501,4 +505,15 @@ extern void	libxfs_dir2_free_log_bests (xfs_trans_t *, xfs_dabuf_t *,
 /* Shared utility routines */
 extern unsigned int	libxfs_log2_roundup(unsigned int i);
 
+
+#include <xfs/xfs_ialloc.h>
+#include <xfs/xfs_rtalloc.h>
+
+#include <xfs/xfs_attr_leaf.h>
+#include <xfs/xfs_quota.h>
+#include <xfs/xfs_trans_space.h>
+#include <xfs/xfs_imap.h>
+#include <xfs/xfs_log.h>
+#include <xfs/xfs_log_priv.h>
+
 #endif	/* __LIBXFS_H__ */
