@@ -1,38 +1,24 @@
 /*
- * Copyright (c) 2003-2005 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2003-2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs/libxfs.h>
-#include "command.h"
-#include "input.h"
+#include <xfs/xfs.h>
+#include <xfs/command.h>
+#include <xfs/input.h>
 #include "init.h"
 #include "io.h"
 
@@ -50,27 +36,129 @@ pwrite_help(void)
 "\n"
 " Writes into a segment of the currently open file, using either a buffer\n"
 " filled with a set pattern (0xcdcdcdcd) or data read from an input file.\n"
-" -S -- use an alternate seed number\n"
-" -i -- specifies an input file from which to source data to write\n"
-" -d -- open the input file for direct IO\n"
-" -s -- skip a number of bytes at the start of the input file\n"
-" -w -- call fdatasync(2) at the end (included in timing results)\n"
-" -W -- call fsync(2) at the end (included in timing results)\n"
 " The writes are performed in sequential blocks starting at offset, with the\n"
-" blocksize tunable using the -b option (default blocksize is 4096 bytes).\n"
+" blocksize tunable using the -b option (default blocksize is 4096 bytes),\n"
+" unless a different write pattern is requested.\n"
+" -S   -- use an alternate seed number for filling the write buffer\n"
+" -i   -- input file, source of data to write (used when writing forward)\n"
+" -d   -- open the input file for direct IO\n"
+" -s   -- skip a number of bytes at the start of the input file\n"
+" -w   -- call fdatasync(2) at the end (included in timing results)\n"
+" -W   -- call fsync(2) at the end (included in timing results)\n"
+" -B   -- write backwards through the range from offset (backwards N bytes)\n"
+" -F   -- write forwards through the range of bytes from offset (default)\n"
+" -R   -- write at random offsets in the specified range of bytes\n"
+" -Z N -- zeed the random number generator (used when writing randomly)\n"
+"         (heh, zorry, the -s/-S arguments were already in use in pwrite)\n"
 "\n"));
 }
 
+static int
+write_random(
+	off64_t		offset,
+	long long	count,
+	unsigned int	seed,
+	long long	*total)
+{
+	off64_t		off, range;
+	ssize_t		bytes;
+	int		ops = 0;
+
+	srandom(seed);
+	if ((bytes = (offset % buffersize)))
+		offset -= bytes;
+	offset = max(0, offset);
+	if ((bytes = (count % buffersize)))
+		count += bytes;
+	count = max(buffersize, count);
+	range = count - buffersize;
+
+	*total = 0;
+	while (count > 0) {
+		off = ((random() % range) / buffersize) * buffersize;
+		bytes = pwrite64(file->fd, buffer, buffersize, off);
+		if (bytes == 0)
+			break;
+		if (bytes < 0) {
+			perror("pwrite64");
+			return -1;
+		}
+		ops++;
+		*total += bytes;
+		if (bytes < buffersize)
+			break;
+		count -= bytes;
+	}
+	return ops;
+}
+
+static int
+write_backward(
+	off64_t		offset,
+	long long	*count,
+	long long	*total)
+{
+	off64_t		end, off = offset;
+	ssize_t		bytes = 0, bytes_requested;
+	long long	cnt = *count;
+	int		ops = 0;
+
+	if ((end = off - cnt) < 0) {
+		cnt += end;	/* subtraction, end is negative */
+		end = 0;
+	}
+	*total = 0;
+	*count = cnt;
+
+	/* Do initial unaligned write if needed */
+	if ((bytes_requested = (off % buffersize))) {
+		bytes_requested = min(cnt, bytes_requested);
+		off -= bytes_requested;
+		bytes = pwrite(file->fd, buffer, bytes_requested, off);
+		if (bytes == 0)
+			return ops;
+		if (bytes < 0) {
+			perror("pwrite64");
+			return -1;
+		}
+		ops++;
+		*total += bytes;
+		if (bytes < bytes_requested)
+			return ops;
+		cnt -= bytes;
+	}
+
+	/* Iterate backward through the rest of the range */
+	while (cnt > end) {
+		bytes_requested = min(cnt, buffersize);
+		off -= bytes_requested;
+		bytes = pwrite64(file->fd, buffer, bytes_requested, off);
+		if (bytes == 0)
+			break;
+		if (bytes < 0) {
+			perror("pwrite64");
+			return -1;
+		}
+		ops++;
+		*total += bytes;
+		if (bytes < bytes_requested)
+			break;
+		cnt -= bytes;
+	}
+	return ops;
+}
+
 static int
 write_buffer(
 	off64_t		offset,
 	long long	count,
-	ssize_t		bs,
+	size_t		bs,
 	int		fd,
 	off64_t		skip,
 	long long	*total)
 {
-	ssize_t		bytes, bytes_requested;
+	size_t		bytes_requested;
+	ssize_t		bytes;
 	long long	bar = min(bs, count);
 	int		ops = 0;
 
@@ -103,30 +191,44 @@ pwrite_f(
 	int		argc,
 	char		**argv)
 {
+	size_t		bsize;
 	off64_t		offset, skip = 0;
-	long long	count, total;
-	unsigned int	seed = 0xcdcdcdcd;
-	int		blocksize, sectsize;
+	long long	count, total, tmp;
+	unsigned int	zeed = 0, seed = 0xcdcdcdcd;
+	size_t		fsblocksize, fssectsize;
 	struct timeval	t1, t2;
 	char		s1[64], s2[64], ts[64];
 	char		*sp, *infile = NULL;
 	int		Cflag, uflag, dflag, wflag, Wflag;
+	int		direction = IO_FORWARD;
 	int		c, fd = -1;
 
 	Cflag = uflag = dflag = wflag = Wflag = 0;
-	init_cvtnum(&blocksize, &sectsize);
-	while ((c = getopt(argc, argv, "b:Cdf:i:s:S:uwW")) != EOF) {
+	init_cvtnum(&fsblocksize, &fssectsize);
+	bsize = fsblocksize;
+
+	while ((c = getopt(argc, argv, "b:Cdf:i:s:S:uwWZ:")) != EOF) {
 		switch (c) {
 		case 'b':
-			blocksize = cvtnum(blocksize, sectsize, optarg);
-			if (blocksize < 0) {
+			tmp = cvtnum(fsblocksize, fssectsize, optarg);
+			if (tmp < 0) {
 				printf(_("non-numeric bsize -- %s\n"), optarg);
 				return 0;
 			}
+			bsize = tmp;
 			break;
 		case 'C':
 			Cflag = 1;
 			break;
+		case 'F':
+			direction = IO_FORWARD;
+			break;
+		case 'B':
+			direction = IO_BACKWARD;
+			break;
+		case 'R':
+			direction = IO_RANDOM;
+			break;
 		case 'd':
 			dflag = 1;
 			break;
@@ -135,7 +237,7 @@ pwrite_f(
 			infile = optarg;
 			break;
 		case 's':
-			skip = cvtnum(blocksize, sectsize, optarg);
+			skip = cvtnum(fsblocksize, fssectsize, optarg);
 			if (skip < 0) {
 				printf(_("non-numeric skip -- %s\n"), optarg);
 				return 0;
@@ -157,25 +259,34 @@ pwrite_f(
 		case 'W':
 			Wflag = 1;
 			break;
+		case 'Z':
+			zeed = strtoul(optarg, &sp, 0);
+			if (!sp || sp == optarg) {
+				printf(_("non-numeric seed -- %s\n"), optarg);
+				return 0;
+			}
+			break;
 		default:
 			return command_usage(&pwrite_cmd);
 		}
 	}
-	if ( ((skip || dflag) && !infile) || (optind != argc - 2))
+	if (((skip || dflag) && !infile) || (optind != argc - 2))
 		return command_usage(&pwrite_cmd);
-	offset = cvtnum(blocksize, sectsize, argv[optind]);
+	if (infile && direction != IO_FORWARD)
+		return command_usage(&pwrite_cmd);
+	offset = cvtnum(fsblocksize, fssectsize, argv[optind]);
 	if (offset < 0) {
 		printf(_("non-numeric offset argument -- %s\n"), argv[optind]);
 		return 0;
 	}
 	optind++;
-	count = cvtnum(blocksize, sectsize, argv[optind]);
+	count = cvtnum(fsblocksize, fssectsize, argv[optind]);
 	if (count < 0) {
 		printf(_("non-numeric length argument -- %s\n"), argv[optind]);
 		return 0;
 	}
 
-	if (alloc_buffer(blocksize, uflag, seed) < 0)
+	if (alloc_buffer(bsize, uflag, seed) < 0)
 		return 0;
 
 	c = IO_READONLY | (dflag ? IO_DIRECT : 0);
@@ -183,7 +294,21 @@ pwrite_f(
 		return 0;
 
 	gettimeofday(&t1, NULL);
-	c = write_buffer(offset, count, blocksize, fd, skip, &total);
+	switch (direction) {
+	case IO_RANDOM:
+		if (!zeed)	/* srandom seed */
+			zeed = time(NULL);
+		c = write_random(offset, count, zeed, &total);
+		break;
+	case IO_FORWARD:
+		c = write_buffer(offset, count, bsize, fd, skip, &total);
+		break;
+	case IO_BACKWARD:
+		c = write_backward(offset, &count, &total);
+		break;
+	default:
+		ASSERT(0);
+	}
 	if (c < 0) {
 		close(fd);
 		return 0;
