@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2004 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
@@ -64,18 +50,16 @@
 
 #define xfs_xlatesb			libxfs_xlate_sb
 #define xfs_xlate_dinode_core		libxfs_xlate_dinode_core
-#if ARCH_CONVERT != ARCH_NOCONVERT
 #define xfs_bmbt_get_all		libxfs_bmbt_get_all
+#if __BYTE_ORDER != __BIG_ENDIAN
 #define xfs_bmbt_disk_get_all		libxfs_bmbt_disk_get_all
 #else
-#define xfs_bmbt_get_all		libxfs_bmbt_get_all
 #define xfs_bmbt_disk_get_all		libxfs_bmbt_get_all
 #endif
 #define xfs_da_hashname			libxfs_da_hashname
 #define xfs_da_log2_roundup		libxfs_da_log2_roundup
 #define xfs_highbit32			libxfs_highbit32
 #define xfs_highbit64			libxfs_highbit64
-#define xfs_attr_leaf_newentsize	libxfs_attr_leaf_newentsize
 #define xfs_alloc_compute_maxlevels	libxfs_alloc_compute_maxlevels
 #define xfs_bmap_compute_maxlevels	libxfs_bmap_compute_maxlevels
 #define xfs_ialloc_compute_maxlevels	libxfs_ialloc_compute_maxlevels
@@ -94,13 +78,17 @@
 #define xfs_dir2_removename		libxfs_dir2_removename
 #define xfs_dir_bogus_removename	libxfs_dir_bogus_removename
 #define xfs_dir2_bogus_removename	libxfs_dir2_bogus_removename
+#define xfs_dir_shortform_to_leaf	libxfs_dir_shortform_to_leaf
+#define xfs_dir2_sf_to_block		libxfs_dir2_sf_to_block
+#define XFS_DIR_SHORTFORM_TO_SINGLE(mp, daargs)	XFS_DIR_IS_V2(mp) ? \
+					libxfs_dir2_sf_to_block(daargs) : \
+					libxfs_dir_shortform_to_leaf(daargs);
 
 #define xfs_mount_common		libxfs_mount_common
 #define xfs_initialize_perag		libxfs_initialize_perag
 #define xfs_rtmount_init		libxfs_rtmount_init
 #define xfs_alloc_fix_freelist		libxfs_alloc_fix_freelist
 #define xfs_iread			libxfs_iread
-#define xfs_ialloc			libxfs_ialloc
 #define xfs_idata_realloc		libxfs_idata_realloc
 #define xfs_idestroy_fork		libxfs_idestroy_fork
 #define xfs_itobp			libxfs_itobp
@@ -123,6 +111,7 @@
 #define xfs_trans_bhold			libxfs_trans_bhold
 #define xfs_trans_alloc			libxfs_trans_alloc
 #define xfs_trans_commit		libxfs_trans_commit
+#define xfs_trans_cancel		libxfs_trans_cancel
 #define xfs_trans_mod_sb		libxfs_trans_mod_sb
 #define xfs_trans_reserve		libxfs_trans_reserve
 #define xfs_trans_get_buf		libxfs_trans_get_buf
@@ -151,6 +140,10 @@
 #define xfs_dir2_data_freescan		libxfs_dir2_data_freescan
 #define xfs_dir2_free_log_bests		libxfs_dir2_free_log_bests
 
+#define xfs_attr_leaf_newentsize	libxfs_attr_leaf_newentsize
+#define xfs_attr_set_int		libxfs_attr_set_int
+#define xfs_attr_remove_int		libxfs_attr_remove_int
+
 
 /*
  * Infrastructure to support building kernel XFS code in user space
@@ -163,10 +156,13 @@
 #define XFS_BUF_ISDONE(bp)		0
 #define XFS_BUF_GETERROR(bp)		0
 #define XFS_BUF_DONE(bp)		((void) 0)
+#define XFS_BUF_STALE(bp)		((void) 0)
+#define XFS_BUF_UNDELAYWRITE(bp)	((void) 0)
 #define XFS_BUF_SET_REF(a,b)		((void) 0)
 #define XFS_BUF_SET_VTYPE(a,b)		((void) 0)
 #define XFS_BUF_SET_VTYPE_REF(a,b,c)	((void) 0)
 #define XFS_BUF_SET_BDSTRAT_FUNC(a,b)	((void) 0)
+#define xfs_incore(bt,blkno,len,lockit)	0
 #define xfs_baread(a,b,c)		((void) 0)	/* no readahead */
 #define xfs_buftrace(x,y)		((void) 0)	/* debug only */
 #define xfs_buf_item_log_debug(bip,a,b)	((void) 0)	/* debug only */
@@ -174,6 +170,14 @@
 #define xfs_buf_relse(bp)		libxfs_putbuf(bp)
 #define xfs_read_buf(mp,devp,blkno,len,f,bpp)	\
 	( *(bpp) = libxfs_readbuf( *(dev_t*)devp, (blkno), (len), 1), 0 )
+#define xfs_buf_get_flags(devp,blkno,len,f)	\
+	( libxfs_getbuf( devp, (blkno), (len) ) )
+#define xfs_bwrite(mp,bp)		libxfs_writebuf((bp), 0)
+
+#define XFS_B_READ			LIBXFS_BREAD
+#define XFS_B_WRITE			LIBXFS_BWRITE
+#define xfs_biomove(bp,off,len,data,f)	libxfs_iomove(bp,off,len,data,f)
+#define xfs_biozero(bp,off,len)		libxfs_iomove(bp,off,len,0,LIBXFS_BZERO)
 
 /* transaction management */
 #define xfs_trans_set_sync(tp)			((void) 0)
@@ -232,6 +236,10 @@
 /* space allocation */
 #define xfs_alloc_search_busy(tp,ag,b,len)		((void) 0)
 #define xfs_alloc_mark_busy(tp,ag,b,len)		((void) 0)
+#define xfs_rotorstep					1
+
+#define xfs_ilock(ip, mode)		((void)0)
+#define xfs_iunlock(ip, mode)		((void)0)
 
 /* anything else */
 #if !defined(__sgi__)
@@ -242,8 +250,12 @@ typedef struct { dev_t dev; } xfs_buftarg_t;
 #undef MASK
 #define NBPP		getpagesize()
 #define STATIC
-#define ATTR_ROOT	0x0002	/* use attrs in root namespace */
-#define ATTR_SECURE	0x0008	/* use attrs in security namespace */
+#define VN_HOLD(vp)
+#define ATTR_ROOT	LIBXFS_ATTR_ROOT
+#define ATTR_SECURE	LIBXFS_ATTR_SECURE
+#define ATTR_CREATE	LIBXFS_ATTR_CREATE
+#define ATTR_REPLACE	LIBXFS_ATTR_REPLACE
+#define ATTR_KERNOTIME	0
 #define ktrace_t	void
 #define m_ddev_targp	m_dev
 #define unlikely(x)	(x)
@@ -254,6 +266,10 @@ typedef struct { dev_t dev; } xfs_buftarg_t;
 #define XFS_ERROR_REPORT(e,l,mp)	((void) 0)
 #define XFS_CORRUPTION_ERROR(e,l,mp,m)	((void) 0)
 #define XFS_TEST_ERROR(expr,a,b,c)	( expr )
+#define XFS_WANT_CORRUPTED_GOTO(expr,l)	\
+		{ if (!(expr)) { error = EFSCORRUPTED; goto l; } }
+#define XFS_WANT_CORRUPTED_RETURN(expr)	\
+		{ if (!(expr)) { return EFSCORRUPTED; } }
 #define TRACE_FREE(s,a,b,x,f)		((void) 0)
 #define TRACE_ALLOC(s,a)		((void) 0)
 #define TRACE_MODAGF(a,b,c)		((void) 0)
@@ -263,9 +279,11 @@ typedef struct { dev_t dev; } xfs_buftarg_t;
 #define XFS_STATS_ADD(count, x)		do { } while (0)
 #define XFS_MOUNT_WSYNC			0	/* ignored in userspace */
 #define XFS_MOUNT_NOALIGN		0	/* ignored in userspace */
-#define XFS_MOUNT_32BITINODES		0x1	/* enforce in userspace */
-#define XFS_MOUNT_32BITINOOPT		0x2	/* enforce in userspace */
+#define XFS_MOUNT_32BITINODES		LIBXFS_MOUNT_32BITINODES
+#define XFS_MOUNT_32BITINOOPT		LIBXFS_MOUNT_32BITINOOPT
+#define XFS_MOUNT_COMPAT_ATTR		LIBXFS_MOUNT_COMPAT_ATTR
 #define XFS_ILOCK_EXCL			0
+#define xfs_sort			qsort
 #define down_read(a)			((void) 0)
 #define up_read(a)			((void) 0)
 #define mrlock(a,b,c)			((void) 0)
@@ -279,12 +297,16 @@ typedef struct { dev_t dev; } xfs_buftarg_t;
 #define __return_address		__builtin_return_address(0)
 #define xfs_btree_reada_bufl(m,fsb,c)	((void) 0)
 #define xfs_btree_reada_bufs(m,fsb,c,x)	((void) 0)
+#define XFS_SB_LOCK(mp)			0
+#define XFS_SB_UNLOCK(mp,s)		((void) 0)
 #undef  XFS_DIR_SHORTFORM_VALIDATE_ONDISK
 #define XFS_DIR_SHORTFORM_VALIDATE_ONDISK(mp,dip) 0
-#define XFS_TRANS_MOD_DQUOT_BYINO(mp,tp,ip,field,delta)
+#define XFS_QM_DQATTACH(mp,ip,flags)	0
+#define XFS_TRANS_MOD_DQUOT_BYINO(mp,tp,ip,field,delta)	do { } while (0)
 #define XFS_TRANS_RESERVE_BLKQUOTA(mp,tp,ip,nblks)	0
-#define XFS_TRANS_UNRESERVE_BLKQUOTA(mp,tp,ip,nblks)
-#define XFS_TRANS_RESERVE_QUOTA_NBLKS(mp,tp,ip,nblks,ninos,fl)
+#define XFS_TRANS_UNRESERVE_BLKQUOTA(mp,tp,ip,nblks)	0
+#define XFS_TRANS_RESERVE_QUOTA_NBLKS(mp,tp,ip,nblks,ninos,fl)	0
+#define XFS_TRANS_UNRESERVE_QUOTA_NBLKS(mp,tp,ip,nblks,ninos,fl)	0
 
 /* These are lifted from the kernel */
 #define get_unaligned(ptr) \
@@ -305,6 +327,7 @@ typedef struct { dev_t dev; } xfs_buftarg_t;
 # define printk(...)			( fprintf(stderr, __VA_ARGS__) )
 #endif
 
+#define rol32(x,y)	(((x) << (y)) | ((x) >> (32 - (y))))
 #define do_mod(a, b)	((a) % (b))
 #define do_div(n,base)	({ \
 	int __res; \
@@ -351,6 +374,8 @@ int  xfs_dilocate (xfs_mount_t *, xfs_trans_t *, xfs_ino_t, xfs_fsblock_t *,
 			int *, int *, uint);
 
 /* xfs_rtalloc.c */
+int  xfs_rtpick_extent (xfs_mount_t *, xfs_trans_t *, xfs_extlen_t,
+			xfs_rtblock_t *);
 int  xfs_rtfree_extent (xfs_trans_t *, xfs_rtblock_t, xfs_extlen_t);
 int  xfs_rtmodify_range (xfs_mount_t *, xfs_trans_t *, xfs_rtblock_t,
 			xfs_extlen_t, int);
@@ -414,10 +439,13 @@ void xfs_idestroy_fork (xfs_inode_t *, int);
 uint xfs_iroundup (uint);
 
 /* xfs_bmap.c */
-xfs_bmbt_rec_t *xfs_bmap_search_extents (xfs_inode_t *ip,
+int  xfs_bmap_local_to_extents (xfs_trans_t *, xfs_inode_t *,
+			xfs_fsblock_t *, xfs_extlen_t, int *, int);
+xfs_bmbt_rec_t *xfs_bmap_search_extents (xfs_inode_t *,
 			xfs_fileoff_t, int, int *, xfs_extnum_t *,
 			xfs_bmbt_irec_t *, xfs_bmbt_irec_t *);
 int  xfs_bmap_read_extents (xfs_trans_t *, xfs_inode_t *, int);
+int  xfs_bmap_add_attrfork (xfs_inode_t *, int, int);
 void xfs_bmap_add_free (xfs_fsblock_t, xfs_filblks_t, xfs_bmap_free_t *,
 			xfs_mount_t *);
 int  xfs_bmap_first_unused (xfs_trans_t *, xfs_inode_t *, xfs_extlen_t,
@@ -446,6 +474,7 @@ int  xfs_bmap_extents_to_btree (xfs_trans_t *, xfs_inode_t *, xfs_fsblock_t *,
 			xfs_bmap_free_t *, xfs_btree_cur_t **, int, int *, int);
 void xfs_bmap_delete_exlist (xfs_inode_t *, xfs_extnum_t, xfs_extnum_t, int);
 xfs_filblks_t xfs_bmap_worst_indlen (xfs_inode_t *, xfs_filblks_t);
+void xfs_bmap_cancel (xfs_bmap_free_t *);
 int  xfs_bmap_isaeof (xfs_inode_t *, xfs_fileoff_t, int, char *);
 void xfs_bmap_insert_exlist (xfs_inode_t *, xfs_extnum_t, xfs_extnum_t,
 			xfs_bmbt_irec_t *, int);
@@ -472,6 +501,8 @@ void xfs_bmbt_log_block (struct xfs_btree_cur *, struct xfs_buf *, int);
 void xfs_bmbt_log_recs (struct xfs_btree_cur *, struct xfs_buf *, int, int);
 int  xfs_bmbt_lookup_eq (struct xfs_btree_cur *, xfs_fileoff_t, xfs_fsblock_t,
 			xfs_filblks_t, int *);
+int  xfs_bmbt_lookup_ge (struct xfs_btree_cur *, xfs_fileoff_t, xfs_fsblock_t,
+			xfs_filblks_t, int *);
 xfs_fsblock_t xfs_bmbt_get_startblock (xfs_bmbt_rec_t *);
 xfs_filblks_t xfs_bmbt_get_blockcount (xfs_bmbt_rec_t *);
 xfs_fileoff_t xfs_bmbt_get_startoff (xfs_bmbt_rec_t *);
@@ -486,7 +517,7 @@ int  xfs_bmbt_update (struct xfs_btree_cur *, xfs_fileoff_t, xfs_fsblock_t,
 			xfs_filblks_t, xfs_exntst_t);
 void xfs_bmbt_to_bmdr (xfs_bmbt_block_t *, int, xfs_bmdr_block_t *, int);
 void xfs_bmdr_to_bmbt (xfs_bmdr_block_t *, int, xfs_bmbt_block_t *, int);
-#if ARCH_CONVERT != ARCH_NOCONVERT
+#if __BYTE_ORDER != __BIG_ENDIAN
 xfs_fileoff_t xfs_bmbt_disk_get_startoff (xfs_bmbt_rec_t *);
 void xfs_bmbt_disk_set_all (xfs_bmbt_rec_t *, xfs_bmbt_irec_t *);
 void xfs_bmbt_disk_set_allf (xfs_bmbt_rec_t *, xfs_fileoff_t, xfs_fsblock_t,
@@ -557,9 +588,11 @@ int  xfs_da_blk_link (xfs_da_state_t *, xfs_da_state_blk_t *,
 xfs_da_state_t *xfs_da_state_alloc (void);
 void xfs_da_state_free (xfs_da_state_t *);
 void xfs_da_state_kill_altpath (xfs_da_state_t *);
+xfs_daddr_t xfs_da_blkno(xfs_dabuf_t *);
 
 /* xfs_dir.c */
 int  xfs_dir_node_addname (xfs_da_args_t *);
+int  xfs_dir_leaf_create (xfs_da_args_t *, xfs_dablk_t, xfs_dabuf_t **);
 int  xfs_dir_leaf_lookup (xfs_da_args_t *);
 int  xfs_dir_node_lookup (xfs_da_args_t *);
 int  xfs_dir_leaf_replace (xfs_da_args_t *);
@@ -576,6 +609,7 @@ int  xfs_dir_leaf_figure_balance (xfs_da_state_t *, xfs_da_state_blk_t *,
 			xfs_da_state_blk_t *, int *, int *);
 void xfs_dir_leaf_moveents (xfs_dir_leafblock_t *, int,
 			xfs_dir_leafblock_t *, int, int, xfs_mount_t *);
+int  xfs_dir_shortform_to_leaf (xfs_da_args_t *);
 
 /* xfs_dir2_leaf.c */
 void xfs_dir2_leaf_check (xfs_inode_t *, xfs_dabuf_t *);
@@ -592,6 +626,7 @@ void xfs_dir2_leafn_check (xfs_inode_t *, xfs_dabuf_t *);
 int  xfs_dir2_leafn_remove (xfs_da_args_t *, xfs_dabuf_t *, int,
 			xfs_da_state_blk_t *, int *);
 int  xfs_dir2_node_addname_int (xfs_da_args_t *, xfs_da_state_blk_t *);
+int  xfs_dir2_sf_to_block (xfs_da_args_t *);
 
 /* xfs_dir2_sf.c */
 void xfs_dir2_sf_check (xfs_da_args_t *);
