@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2003 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2003,2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "logprint.h"
@@ -82,6 +68,7 @@ char *trans_type[] = {
 	"GROWFSRT_ZERO",
 	"GROWFSRT_FREE",
 	"SWAPEXT",
+	"SB_COUNT",
 };
 
 typedef struct xlog_split_item {
@@ -882,7 +869,7 @@ print_lsn(xfs_caddr_t	string,
 	  xfs_arch_t    arch)
 {
     printf("%s: %u,%u", string,
-	    CYCLE_LSN(*lsn, arch), BLOCK_LSN(*lsn, arch));
+	    CYCLE_LSN(INT_GET(*lsn, arch)), BLOCK_LSN(INT_GET(*lsn, arch)));
 }
 
 
@@ -927,8 +914,9 @@ xlog_print_record(int			  fd,
     }
     /* Did we overflow the end? */
     if (*read_type == FULL_READ &&
-	BLOCK_LSN(rhead->h_lsn, ARCH_CONVERT)+BTOBB(read_len) >= logBBsize) {
-	*read_type = BBTOB(logBBsize-BLOCK_LSN(rhead->h_lsn, ARCH_CONVERT)-1);
+	BLOCK_LSN(INT_GET(rhead->h_lsn, ARCH_CONVERT)) + BTOBB(read_len) >=
+		logBBsize) {
+	*read_type = BBTOB(logBBsize - BLOCK_LSN(INT_GET(rhead->h_lsn, ARCH_CONVERT))-1);
 	*partial_buf = buf;
 	return PARTIAL_READ;
     }
@@ -946,7 +934,7 @@ xlog_print_record(int			  fd,
      * Unpack the data, by putting the saved cycle-data back
      * into the first word of each BB.
      * Do some checks.
-     */	
+     */
     buf = ptr;
     for (i = 0; ptr < buf + read_len; ptr += BBSIZE, i++) {
 	xlog_rec_header_t *rechead = (xlog_rec_header_t *)ptr;
@@ -1094,7 +1082,7 @@ xlog_print_rec_head(xlog_rec_header_t *head, int *len)
     if (print_no_print)
 	    return INT_GET(head->h_num_logops, ARCH_CONVERT);
 
-    if (INT_ISZERO(head->h_magicno, ARCH_CONVERT))
+    if (!head->h_magicno)
 	return ZEROED_LOG;
 
     if (INT_GET(head->h_magicno, ARCH_CONVERT) != XLOG_HEADER_MAGIC_NUM) {
@@ -1105,11 +1093,8 @@ xlog_print_rec_head(xlog_rec_header_t *head, int *len)
     }
 
     /* check for cleared blocks written by xlog_clear_stale_blocks() */
-    if (INT_ISZERO(head->h_len, ARCH_CONVERT) &&
-	INT_ISZERO(head->h_chksum, ARCH_CONVERT) &&
-	INT_ISZERO(head->h_prev_block, ARCH_CONVERT) &&
-	INT_ISZERO(head->h_num_logops, ARCH_CONVERT) &&
-	INT_ISZERO(head->h_size, ARCH_CONVERT))
+    if (!head->h_len && !head->h_chksum && !head->h_prev_block &&
+	!head->h_num_logops && !head->h_size)
 	return CLEARED_BLKS;
 
     datalen=INT_GET(head->h_len, ARCH_CONVERT);
