@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2005 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * Copyright (c) 2000-2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- * 
- * http://www.sgi.com 
- * 
- * For further information regarding this notice, see: 
- * 
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ *
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <xfs/libxfs.h>
@@ -84,6 +70,12 @@ char	*dopts[] = {
 	"sectsize",
 #define D_NOALIGN	12
 	"noalign",
+#define D_RTINHERIT	13
+	"rtinherit",
+#define D_PROJINHERIT	14
+	"projinherit",
+#define D_EXTSZINHERIT	15
+	"extszinherit",
 	NULL
 };
 
@@ -98,6 +90,8 @@ char	*iopts[] = {
 	"perblock",
 #define	I_SIZE		4
 	"size",
+#define	I_ATTR		5
+	"attr",
 	NULL
 };
 
@@ -295,27 +289,17 @@ check_overwrite(
 	return 0;
 }
 
-static xfs_dfsbno_t
-fixup_log_stripe(
-	xfs_mount_t	*mp,
+static void
+fixup_log_stripe_unit(
 	int		lsflag,
-	xfs_dfsbno_t	logstart,
-	__uint64_t	agsize,
 	int		sunit,
 	xfs_drfsbno_t	*logblocks,
-	int		blocklog,
-	int		*lalign)
+	int		blocklog)
 {
 	__uint64_t	tmp_logblocks;
 
-	if ((logstart % sunit) != 0) {
-		logstart = ((logstart + (sunit - 1))/sunit) * sunit;
-		*lalign = 1;
-	}
-
-	/* 
-	 * Make sure that the log size is a multiple of the
-	 * stripe unit
+	/*
+	 * Make sure that the log size is a multiple of the stripe unit
 	 */
 	if ((*logblocks % sunit) != 0) {
 		if (!lsflag) {
@@ -331,13 +315,31 @@ fixup_log_stripe(
 			}
 			*logblocks = tmp_logblocks;
 		} else {
-			fprintf(stderr,
-				_("internal log size %lld is not a multiple "
-				"of the log stripe unit %d\n"),
+			fprintf(stderr, _("log size %lld is not a multiple "
+					  "of the log stripe unit %d\n"),
 				(long long) *logblocks, sunit);
 			usage();
 		}
 	}
+}
+
+static xfs_dfsbno_t
+fixup_internal_log_stripe(
+	xfs_mount_t	*mp,
+	int		lsflag,
+	xfs_dfsbno_t	logstart,
+	__uint64_t	agsize,
+	int		sunit,
+	xfs_drfsbno_t	*logblocks,
+	int		blocklog,
+	int		*lalign)
+{
+	if ((logstart % sunit) != 0) {
+		logstart = ((logstart + (sunit - 1))/sunit) * sunit;
+		*lalign = 1;
+	}
+
+	fixup_log_stripe_unit(lsflag, sunit, logblocks, blocklog);
 
 	if (*logblocks > agsize - XFS_FSB_TO_AGBNO(mp, logstart)) {
 		fprintf(stderr,
@@ -513,10 +515,11 @@ main(
 	xfs_agnumber_t		agno;
 	__uint64_t		agsize;
 	xfs_alloc_rec_t		*arec;
+	int			attrversion;
 	xfs_btree_sblock_t	*block;
 	int			blflag;
 	int			blocklog;
-	int			blocksize;
+	unsigned int		blocksize;
 	int			bsflag;
 	int			bsize;
 	xfs_buf_t		*buf;
@@ -535,6 +538,7 @@ main(
 	int			dswidth;
 	int			extent_flagging;
 	int			force_overwrite;
+	struct fsxattr		fsx;
 	int			iaflag;
 	int			ilflag;
 	int			imaxpct;
@@ -587,8 +591,9 @@ main(
 	char			*rtfile;
 	char			*rtsize;
 	xfs_sb_t		*sbp;
+	int			sectoralign;
 	int			sectorlog;
-	int			sectorsize;
+	unsigned int		sectorsize;
 	int			slflag;
 	int			ssflag;
 	__uint64_t		tmp_agsize;
@@ -603,6 +608,7 @@ main(
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 
+	attrversion = 0;
 	blflag = bsflag = slflag = ssflag = lslflag = lssflag = 0;
 	blocklog = blocksize = 0;
 	sectorlog = lsectorlog = XFS_MIN_SECTORSIZE_LOG;
@@ -625,6 +631,7 @@ main(
 	extent_flagging = 1;
 	force_overwrite = 0;
 	worst_freelist = 0;
+	bzero(&fsx, sizeof(fsx));
 
 	bzero(&xi, sizeof(xi));
 	xi.notvolok = 1;
@@ -840,6 +847,24 @@ main(
 						libxfs_highbit32(sectorsize);
 					ssflag = 1;
 					break;
+				case D_RTINHERIT:
+					fsx.fsx_xflags |= \
+						XFS_DIFLAG_RTINHERIT;
+					break;
+				case D_PROJINHERIT:
+					if (!value)
+						reqval('d', dopts, D_PROJINHERIT);
+					fsx.fsx_projid = atoi(value);
+					fsx.fsx_xflags |= \
+						XFS_DIFLAG_PROJINHERIT;
+					break;
+				case D_EXTSZINHERIT:
+					if (!value)
+						reqval('d', dopts, D_EXTSZINHERIT);
+					fsx.fsx_extsize = atoi(value);
+					fsx.fsx_xflags |= \
+						XFS_DIFLAG_EXTSZINHERIT;
+					break;
 				default:
 					unknown('d', value);
 				}
@@ -920,6 +945,14 @@ main(
 					inodelog = libxfs_highbit32(isize);
 					isflag = 1;
 					break;
+				case I_ATTR:
+					if (!value)
+						reqval('i', iopts, I_ATTR);
+					c = atoi(value);
+					if (c < 0 || c > 2)
+						illegal(value, "i attr");
+					attrversion = c;
+					break;
 				default:
 					unknown('i', value);
 				}
@@ -1270,6 +1303,8 @@ main(
 
 	/*
 	 * Blocksize and sectorsize first, other things depend on them
+	 * For RAID4/5/6 we want to align sector size and block size,
+	 * so we need to start with the device geometry extraction too.
 	 */
 	if (!blflag && !bsflag) {
 		blocklog = XFS_DFL_BLOCKSIZE_LOG;
@@ -1279,6 +1314,20 @@ main(
 		fprintf(stderr, _("illegal block size %d\n"), blocksize);
 		usage();
 	}
+
+	sectoralign = 0;
+	xlv_dsunit = xlv_dswidth = 0;
+	if (!nodsflag && !xi.disfile)
+		get_subvol_stripe_wrapper(dfile, SVTYPE_DATA,
+				&xlv_dsunit, &xlv_dswidth, &sectoralign);
+	if (sectoralign) {
+		sectorsize = blocksize;
+		sectorlog = libxfs_highbit32(sectorsize);
+		if (loginternal) {
+			lsectorsize = sectorsize;
+			lsectorlog = sectorlog;
+		}
+	}
 	if (sectorsize < XFS_MIN_SECTORSIZE ||
 	    sectorsize > XFS_MAX_SECTORSIZE || sectorsize > blocksize) {
 		fprintf(stderr, _("illegal sector size %d\n"), sectorsize);
@@ -1442,9 +1491,9 @@ main(
 
 		dummy1 = rswidth = 0;
 
-		if (!norsflag && !xi.risfile)
-			get_subvol_stripe_wrapper(dfile, SVTYPE_RT, &dummy1, 
-						  &rswidth);
+		if (!norsflag && !xi.risfile && !(!rtsize && xi.disfile))
+			get_subvol_stripe_wrapper(dfile, SVTYPE_RT, &dummy1,
+						  &rswidth, &dummy1);
 
 		/* check that rswidth is a multiple of fs blocksize */
 		if (!norsflag && rswidth && !(BBTOB(rswidth) % blocksize)) {
@@ -1687,10 +1736,6 @@ _("size %s specified for log subvolume is too large, maximum is %lld blocks\n"),
 
 	validate_ag_geometry(blocklog, dblocks, agsize, agcount);
 
-	xlv_dsunit = xlv_dswidth = 0;
-	if (!nodsflag && !xi.disfile)
-		get_subvol_stripe_wrapper(dfile, SVTYPE_DATA,
-						&xlv_dsunit, &xlv_dswidth);
 	if (!nodsflag && dsunit) {
 		if (xlv_dsunit && xlv_dsunit != dsunit) {
 			fprintf(stderr,
@@ -1874,24 +1919,26 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 		 * Align the logstart at stripe unit boundary.
 		 */
 		if (lsunit) {
-			logstart = fixup_log_stripe(mp, lsflag, logstart,
-					agsize, lsunit, &logblocks, blocklog,
-					&lalign);
+			logstart = fixup_internal_log_stripe(mp,
+					lsflag, logstart, agsize, lsunit,
+					&logblocks, blocklog, &lalign);
 		} else if (dsunit) {
-			logstart = fixup_log_stripe(mp, lsflag, logstart,
-					agsize, dsunit, &logblocks, blocklog,
-					&lalign);
+			logstart = fixup_internal_log_stripe(mp,
+					lsflag, logstart, agsize, dsunit,
+					&logblocks, blocklog, &lalign);
 		}
-		min_logblocks = max_tr_res * XFS_MIN_LOG_FACTOR;
-		min_logblocks = MAX(min_logblocks, XFS_MIN_LOG_BLOCKS);
-		validate_log_size(logblocks, blocklog, min_logblocks);
-	} else
+	} else {
 		logstart = 0;
+		if (lsunit)
+			fixup_log_stripe_unit(lsflag, lsunit,
+					&logblocks, blocklog);
+	}
+	validate_log_size(logblocks, blocklog, min_logblocks);
 
 	if (!qflag || Nflag) {
 		printf(_(
 		   "meta-data=%-22s isize=%-6d agcount=%lld, agsize=%lld blks\n"
-		   "         =%-22s sectsz=%-5u\n"
+		   "         =%-22s sectsz=%-5u attr=%u\n"
 		   "data     =%-22s bsize=%-6u blocks=%llu, imaxpct=%u\n"
 		   "         =%-22s sunit=%-6u swidth=%u blks, unwritten=%u\n"
 		   "naming   =version %-14u bsize=%-6u\n"
@@ -1899,7 +1946,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 		   "         =%-22s sectsz=%-5u sunit=%d blks\n"
 		   "realtime =%-22s extsz=%-6d blocks=%lld, rtextents=%lld\n"),
 			dfile, isize, (long long)agcount, (long long)agsize,
-			"", sectorsize,
+			"", sectorsize, attrversion,
 			"", blocksize, (long long)dblocks,
 				imflag ? imaxpct : XFS_DFL_IMAXIMUM_PCT,
 			"", dsunit, dswidth, extent_flagging,
@@ -1969,13 +2016,16 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 	}
 	sbp->sb_versionnum =
 		XFS_SB_VERSION_MKFS(iaflag, dsunit != 0, extent_flagging,
-			dirversion == 2, logversion == 2,
-			(sectorsize != BBSIZE || lsectorsize != BBSIZE), 0);
-	sbp->sb_features2 = XFS_SB_VERSION2_MKFS(0);
+			dirversion == 2, logversion == 2, attrversion == 1,
+			(sectorsize != BBSIZE || lsectorsize != BBSIZE),
+			(0/*mmr*/|| 0/*lazy_sb_counters*/ || attrversion == 2));
+	sbp->sb_features2 = XFS_SB_VERSION2_MKFS(0/*mmr*/,0/*lazy_sb_counters*/,
+				attrversion == 2);
+
 	/*
-	 * Zero out the beginning of the device, to obliterate any old 
-	 * filesystem signatures out there.  This should take care of 
-	 * swap (somewhere around the page size), jfs (32k), 
+	 * Zero out the beginning of the device, to obliterate any old
+	 * filesystem signatures out there.  This should take care of
+	 * swap (somewhere around the page size), jfs (32k),
 	 * ext[2,3] and reiserfs (64k) - and hopefully all else.
 	 */
 	buf = libxfs_getbuf(xi.ddev, 0, BTOBB(WHACK_SIZE));
@@ -1985,8 +2035,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 	/* OK, now write the superblock */
 	buf = libxfs_getbuf(xi.ddev, XFS_SB_DADDR, XFS_FSS_TO_BB(mp, 1));
 	bzero(XFS_BUF_PTR(buf), sectorsize);
-	libxfs_xlate_sb(XFS_BUF_PTR(buf), sbp, -1, ARCH_CONVERT,
-			XFS_SB_ALL_BITS);
+	libxfs_xlate_sb(XFS_BUF_PTR(buf), sbp, -1, XFS_SB_ALL_BITS);
 	libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 
 	/*
@@ -2037,8 +2086,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 				XFS_AG_DADDR(mp, agno, XFS_SB_DADDR),
 				XFS_FSS_TO_BB(mp, 1));
 		bzero(XFS_BUF_PTR(buf), sectorsize);
-		libxfs_xlate_sb(XFS_BUF_PTR(buf), sbp, -1, ARCH_CONVERT,
-				XFS_SB_ALL_BITS);
+		libxfs_xlate_sb(XFS_BUF_PTR(buf), sbp, -1, XFS_SB_ALL_BITS);
 		libxfs_writebuf(buf, LIBXFS_EXIT_ON_FAILURE);
 
 		/*
@@ -2242,7 +2290,7 @@ an AG size that is one stripe unit smaller, for example %llu.\n"),
 	 * Allocate the root inode and anything else in the proto file.
 	 */
 	mp->m_rootip = NULL;
-	parseproto(mp, NULL, &protostring, NULL);
+	parse_proto(mp, &fsx, &protostring);
 
 	/*
 	 * Protect ourselves against possible stupidity
@@ -2381,8 +2429,8 @@ isdigits(
 
 long long
 cvtnum(
-	int		blocksize,
-	int		sectorsize,
+	unsigned int	blocksize,
+	unsigned int	sectorsize,
 	char		*s)
 {
 	long long	i;
@@ -2428,7 +2476,7 @@ usage( void )
 /* data subvol */	[-d agcount=n,agsize=n,file,name=xxx,size=num,\n\
 			    (sunit=value,swidth=value|su=num,sw=num),\n\
 			    sectlog=n|sectsize=num,unwritten=0|1]\n\
-/* inode size */	[-i log=n|perblock=n|size=num,maxpct=n]\n\
+/* inode size */	[-i log=n|perblock=n|size=num,maxpct=n,attr=0|1|2]\n\
 /* log subvol */	[-l agnum=n,internal,size=num,logdev=xxx,version=n\n\
 			    sunit=value|su=num,sectlog=n|sectsize=num]\n\
 /* label */		[-L label (maximum 12 characters)]\n\
