@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2002,2004 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2002,2004-2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <libxfs.h>
@@ -172,7 +158,7 @@ process_shortform_attr(
 
 	*repair = 0;
 
-	asf = (xfs_attr_shortform_t *) XFS_DFORK_APTR_ARCH(dip, ARCH_CONVERT);
+	asf = (xfs_attr_shortform_t *) XFS_DFORK_APTR(dip);
 
 	/* Assumption: hdr.totsize is less than a leaf block and was checked
 	 * by lclinode for valid sizes. Check the count though.
@@ -987,11 +973,11 @@ process_attributes(
 	xfs_attr_shortform_t *asf;
 
 	dinoc = &dip->di_core;
-	asf = (xfs_attr_shortform_t *) XFS_DFORK_APTR_ARCH(dip, ARCH_CONVERT);
+	asf = (xfs_attr_shortform_t *) XFS_DFORK_APTR(dip);
 
 	if (dinoc->di_aformat == XFS_DINODE_FMT_LOCAL) {
 		ASSERT(INT_GET(asf->hdr.totsize, ARCH_CONVERT) <=
-			XFS_DFORK_ASIZE_ARCH(dip, mp, ARCH_CONVERT));
+			XFS_DFORK_ASIZE(dip, mp));
 		err = process_shortform_attr(ino, dip, repair);
 	} else if (dinoc->di_aformat == XFS_DINODE_FMT_EXTENTS ||
 		   dinoc->di_aformat == XFS_DINODE_FMT_BTREE)  {
